#
# Pseudo ranking data generator
#

import numpy as np
import math, os
import itertools

#### Parameters ####

num_faculty = 5   # This will be the number of samples in each series
num_students = 100 # This will be the number of series.

rankbase = list(range(num_faculty)) # the baseline ranking

distfunc = 3 # distance function, 1:spearmanRankCorr, 2:spearmanFootrule, 3:spearmanWeighted

theta = 1    # distance parameter (must be positive)

# disctionary for distance functions
dic_dist = {1:"spearmanRankCorr",2:"spearmanFootrule",3:"spearmanWeighted"}

# the csv file name for the output
file_out = 'Sample_N' + str(num_faculty) + '_S' + str(num_students) \
    + '_theta' + str(theta) + '_' + dic_dist[distfunc] + '.csv'


#### Distance measure ####

# Spearman Rank Correlation
def spearmanRankCorr(l1, l2):
    m = 0
    for i in range(len(l1)):
        m += (l1[i]-l2[i])**2
    return m

# Spearman Footrule
def spearmanFootrule(l1, l2):
    m = 0
    for i in range(len(l1)):
        m += abs(l1[i]-l2[i])
    return m

# Spearman Weighted Footrule
# This is in Chapter 16 of Langville and Meyer (2012).
def spearmanWeightedFootrule(l1, l2):
    m = 0
    for i in range(len(l1)):
        m += abs(l1[i]-l2[i]) / (min(l1[i], l2[i])+1)
    return m


#### Main Program ####

dic_dist2 = {1:spearmanRankCorr,2:spearmanFootrule,3:spearmanWeightedFootrule}

RP = list(itertools.permutations(rankbase))

nump = len(RP)
dist = [0] * nump
for i in range(len(RP)):
    # distance between RP[i] and rankbase
    dist[i] = math.exp(-theta*dic_dist2[distfunc](rankbase, RP[i]))

Z = sum(dist)
CDF = np.zeros(nump)
x = 0
for i in range(len(RP)):
    # distance between RP[i] and rankbase
    CDF[i] = (x+dist[i])/ Z
    x += dist[i]

print(nump, Z, x)

r = np.random.uniform(size=num_students)
listranking = [""] * num_students

for i in range(num_students):
    j = CDF.searchsorted(r[i])
    listranking[i] = "s" + str(i+1) + ", "
    listranking[i] += ', '.join([str(x+1) for x in RP[j]])


# save the generated data in a csv file.
with open(file_out, 'wt') as f:
    for i in range(num_students):
        f.write(listranking[i]+'\n')
